(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory() :
	typeof define === 'function' && define.amd ? define(factory) :
	(global.bmPendingQueue = factory());
}(this, (function () { 'use strict';

var commonjsGlobal = typeof window !== 'undefined' ? window : typeof global !== 'undefined' ? global : typeof self !== 'undefined' ? self : {};

// Unique ID creation requires a high quality random # generator.  In the
// browser this is a little complicated due to unknown quality of Math.random()
// and inconsistent support for the `crypto` API.  We do the best we can via
// feature-detection
var rng$1;

var crypto = commonjsGlobal.crypto || commonjsGlobal.msCrypto; // for IE 11
if (crypto && crypto.getRandomValues) {
  // WHATWG crypto RNG - http://wiki.whatwg.org/wiki/Crypto
  var rnds8 = new Uint8Array(16);
  rng$1 = function whatwgRNG() {
    crypto.getRandomValues(rnds8);
    return rnds8;
  };
}

if (!rng$1) {
  // Math.random()-based (RNG)
  //
  // If all else fails, use Math.random().  It's fast, but is of unspecified
  // quality.
  var rnds = new Array(16);
  rng$1 = function rng$1() {
    for (var i = 0, r; i < 16; i++) {
      if ((i & 0x03) === 0) r = Math.random() * 0x100000000;
      rnds[i] = r >>> ((i & 0x03) << 3) & 0xff;
    }

    return rnds;
  };
}

var rngBrowser = rng$1;

/**
 * Convert array of 16 byte values to UUID string format of the form:
 * XXXXXXXX-XXXX-XXXX-XXXX-XXXX-XXXXXXXXXXXX
 */
var byteToHex = [];
for (var i = 0; i < 256; ++i) {
  byteToHex[i] = (i + 0x100).toString(16).substr(1);
}

function bytesToUuid$1(buf, offset) {
  var i = offset || 0;
  var bth = byteToHex;
  return bth[buf[i++]] + bth[buf[i++]] + bth[buf[i++]] + bth[buf[i++]] + '-' + bth[buf[i++]] + bth[buf[i++]] + '-' + bth[buf[i++]] + bth[buf[i++]] + '-' + bth[buf[i++]] + bth[buf[i++]] + '-' + bth[buf[i++]] + bth[buf[i++]] + bth[buf[i++]] + bth[buf[i++]] + bth[buf[i++]] + bth[buf[i++]];
}

var bytesToUuid_1 = bytesToUuid$1;

var rng = rngBrowser;
var bytesToUuid = bytesToUuid_1;

function v4(options, buf, offset) {
  var i = buf && offset || 0;

  if (typeof options == 'string') {
    buf = options == 'binary' ? new Array(16) : null;
    options = null;
  }
  options = options || {};

  var rnds = options.random || (options.rng || rng)();

  // Per 4.4, set bits for version and `clock_seq_hi_and_reserved`
  rnds[6] = rnds[6] & 0x0f | 0x40;
  rnds[8] = rnds[8] & 0x3f | 0x80;

  // Copy bytes to buffer, if provided
  if (buf) {
    for (var ii = 0; ii < 16; ++ii) {
      buf[i + ii] = rnds[ii];
    }
  }

  return buf || bytesToUuid(rnds);
}

var v4_1 = v4;

var uuid = v4_1;

function uuidService$1() {
  return function () {
    return uuid();
  };
}

var uuidService_1 = uuidService$1;

pendingQueueService$1.$inject = ['$rootScope', '$q', '$localForage', 'uuidService'];
function pendingQueueService$1($rootScope, $q, $localForage, uuidService) {
  this.setItem = function saveToPendingQueue(_ref) {
    var url = _ref.url,
        data = _ref.data,
        headers = _ref.headers,
        params = _ref.params,
        method = _ref.method;

    var uuid = data._uuid || uuidService();
    data._uuid = uuid; // make sure that the _uuid prop exists

    return $localForage.setItem(uuid, {
      request: { url: url, data: data, headers: headers, params: params, method: method },
      dateCreated: new Date().getTime()
    }).then(function (item) {
      $rootScope.$broadcast('bmPendingQueueAdd', item);
      return item;
    });
  };

  this.getItem = function getFromPendingQueue(uuid) {
    return $localForage.getItem(uuid);
  };

  this.getEarliest = function getEarliest() {
    var currDate = new Date().getTime();
    var oldestForm = void 0;

    return $localForage.length().then(function (count) {
      return $localForage.iterate(function (form, key, loopCount) {
        if (form.dateCreated < currDate) {
          currDate = form.dateCreated;
          oldestForm = form;
        }

        if (loopCount >= count) {
          return oldestForm;
        }
      });
    });
  };

  this.removeItem = function removeFromPendingQueue(uuid) {
    return $localForage.pull(uuid).then(function (result) {
      $rootScope.$broadcast('bmPendingQueueRemove', result);
      return result;
    });
  };

  this.clear = function clearPendingQueue() {
    return $localForage.clear().then(function (result) {
      $rootScope.$broadcast('bmPendingQueueRemove', result);
      return result;
    });
  };

  this.setResponse = function setResponseOnItem(uuid, response) {
    return this.getItem(uuid).then(function (data) {
      data.response = response;
      $rootScope.$broadcast('bmPendingQueueItemUpdate', data);
      return data;
    }).then(function (data) {
      return $localForage.setItem(uuid, data);
    });
  };

  this.iterate = function iterate(iter) {
    return $localForage.iterate(iter);
  };

  this.length = function length() {
    return $localForage.length();
  };
}

var pendingQueueService_1 = pendingQueueService$1;

pendingQueueInteceptor.$inject = ['$window', '$q', 'bmPendingQueueService'];
function pendingQueueInteceptor($window, $q, bmPendingQueueService) {
  var isPOSTorPUT = function isPOSTorPUT(method) {
    return ['POST', 'PUT'].indexOf(method) > -1;
  };
  var isFormData = function isFormData(contentType) {
    return contentType.toLowerCase().indexOf('application/x-www-form-urlencoded') > -1;
  };
  var isJSONData = function isJSONData(contentType) {
    return contentType.toLowerCase().indexOf('json') > -1;
  };

  var isForm = function isForm(config) {
    return isPOSTorPUT(config.method.toUpperCase()) && (isFormData(config.headers['Content-Type']) || isJSONData(config.headers['Content-Type']));
  };

  return {
    // request errors are thrown when the server name cant be resolved or if you are offline
    // as well as when another interceptor rejects the request
    requestError: function PendingQueueRequestError(rejection) {
      if (rejection.config && isForm(rejection.config)) {
        var cleanedResponse = {
          data: rejection.data,
          status: rejection.status,
          statusText: rejection.statusText || (!$window.navigator.onLine ? 'offline' : 'unknown')
        };

        return bmPendingQueueService.setItem(rejection.config).then(function () {
          return bmPendingQueueService.setResponse(rejection.config.data._uuid, cleanedResponse);
        }).then(function () {
          return $q.reject(rejection);
        });
      }

      return $q.reject(rejection);
    },

    response: function PendingQueueResponse(response) {
      if (!response.config || !isForm(response.config) || !response.config.data._uuid) {
        return response;
      }

      return bmPendingQueueService.removeItem(response.config.data._uuid).then(function () {
        return response;
      });
    },

    responseError: function PendingQueueResponseError(rejection) {
      if (rejection.config && isForm(rejection.config)) {
        var cleanedResponse = {
          data: rejection.data,
          status: rejection.status,
          statusText: rejection.statusText
        };

        return bmPendingQueueService.setItem(rejection.config).then(function () {
          return bmPendingQueueService.setResponse(rejection.config.data._uuid, cleanedResponse);
        }).then(function () {
          return $q.reject(rejection);
        });
      }

      return $q.reject(rejection);
    }
  };
}

var pendingQueueInterceptor$1 = pendingQueueInteceptor;

httpDecorator.$inject = ['$delegate', '$q', '$timeout', '$rootScope', 'bmPendingQueueService'];
function httpDecorator($delegate, $q, $timeout, $rootScope, bmPendingQueueService) {
  var sendNextPending = function sendNextPending(response) {
    // we dont want to make the user wait for all the pending
    // queue items to be sent and processed
    $rootScope.$evalAsync(function () {
      var deferred = [];
      bmPendingQueueService.iterate(function (item) {
        deferred.push($delegate(item.request));
      }).then(function () {
        if (!deferred.length) {
          return;
        }

        $q.all(deferred).then(function (result) {
          return $rootScope.$broadcast('bmPendingQueueSuccess', result);
        }).catch(function (result) {
          return $rootScope.$broadcast('bmPendingQueueFail', result);
        });
      });
    });

    return response;
  };

  function pendingQueueHTTP() {
    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    return $delegate.apply($delegate, args).then(sendNextPending);
  }

  // make our decorator delegate to the $http methods
  // and properties

  var _loop = function _loop(prop) {
    if ($delegate.hasOwnProperty(prop)) {
      if (angular.isFunction($delegate[prop])) {
        pendingQueueHTTP[prop] = function () {
          for (var _len2 = arguments.length, args = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
            args[_key2] = arguments[_key2];
          }

          return $delegate[prop].apply($delegate, args).then(sendNextPending);
        };
      } else {
        Object.defineProperty(pendingQueueHTTP, prop, {
          get: function get() {
            return $delegate[prop];
          },
          set: function set(val) {
            return $delegate[prop] = val;
          }
        });
      }
    }
  };

  for (var prop in $delegate) {
    _loop(prop);
  }

  // access to the original $http
  pendingQueueHTTP.$delegate = $delegate;

  return pendingQueueHTTP;
}

var httpDecorator_1 = httpDecorator;

PendingQueueListController.$inject = ['$scope', '$q', '$sce', '$window', 'bmPendingQueueService'];
function PendingQueueListController($scope, $q, $sce, $window, bmPendingQueueService) {
  var $ctrl = this;
  var watchers = [];
  var fetching = false;

  $ctrl.$onInit = function () {
    fetching = false;
    $ctrl.pendingQueue = [];
    if (!$ctrl.displayRemove) {
      $ctrl.displayRemove = '&#x2718;';
    }
    $ctrl.displayRemove = $sce.trustAsHtml($ctrl.displayRemove);
    watchers.push($scope.$on('bmPendingQueueRemove', $ctrl.getQueue));
    watchers.push($scope.$on('bmPendingQueueAdd', $ctrl.getQueue));
    $ctrl.getQueue();
  };

  $ctrl.$onDestroy = function () {
    watchers.forEach(function (w) {
      return w();
    });
  };

  $ctrl.selectItem = function (item) {
    $ctrl.onSelectItem && $ctrl.onSelectItem({ item: item });
  };

  var remove = function remove(uuid) {
    return bmPendingQueueService.removeItem(uuid).then($ctrl.getQueue);
  };
  $ctrl.removeItem = function (uuid) {
    if ($ctrl.onRemoveItem) {
      return bmPendingQueueService.getItem(uuid).then(function (item) {
        return $ctrl.onRemoveItem({ item: item });
      }).then(remove).catch(function () {
        return false;
      });
    }

    if ($window.confirm('Are you sure you want to remove this item?')) {
      return remove(uuid);
    }
  };

  $ctrl.getQueue = function () {
    if (fetching) {
      return;
    }
    fetching = true;
    $ctrl.pendingQueue.length = 0;
    return bmPendingQueueService.iterate(function (item) {
      $ctrl.pendingQueue.push(item.request.data);
    }).then(function () {
      return fetching = false;
    });
  };
}

var pendingQueueListComponent = {
  controller: PendingQueueListController,
  controllerAs: 'PendingQueueListCtrl',
  template: '<div class="bm-pending-queue">\n  <ul class="bm-pending-queue__list">\n    <li class="bm-pending-queue__list-item" ng-repeat="item in PendingQueueListCtrl.pendingQueue">\n      <a  class="bm-pending-queue__select-item"\n          title="Select item"\n          href=""\n          ng-click="PendingQueueListCtrl.selectItem(item)">{{item[PendingQueueListCtrl.displayKey]}}</a>\n      <a class="bm-pending-queue__remove-item bm-button__icon"\n         href=""\n         title="Remove Item"\n         ng-click="PendingQueueListCtrl.removeItem(item._uuid)"\n         ng-bind-html="PendingQueueListCtrl.displayRemove"></a>\n    </li>\n  </ul>\n</div>\n',
  bindings: {
    displayKey: '@',
    displayRemove: '@?',
    onSelectItem: '&?',
    onRemoveItem: '&?'
  }
};

var uuidService = uuidService_1;
var pendingQueueService = pendingQueueService_1;
var pendingQueueInterceptor = pendingQueueInterceptor$1;
var $httpDecorator = httpDecorator_1;
var queueListComponent = pendingQueueListComponent;

angular.module('bmPendingQueue', ['LocalForageModule']).constant('dbName', 'bmOfflineStorage').constant('pendingQueueTable', 'pendingQueue').config(['$httpProvider', '$localForageProvider', 'dbName', 'pendingQueueTable', function ($httpProvider, $localForageProvider, dbName, pendingQueueTable) {
  $localForageProvider.config({
    name: dbName,
    storeName: pendingQueueTable,
    description: 'Blink Mobile Technologies Forms Pending Queue'
  });

  $httpProvider.interceptors.push('bmPendingQueueInterceptor');
}]).decorator('$http', $httpDecorator).factory('bmPendingQueueInterceptor', pendingQueueInterceptor).service('uuidService', uuidService).service('bmPendingQueueService', pendingQueueService).component('pendingQueueList', queueListComponent);

var pendingQueueModule = {};

return pendingQueueModule;

})));
